'''
Created on Nov 25, 2019
Sample structure of run file run.py

@author: cxchu
@editor: ghoshs
'''

import sys
import csv
from dateutil import parser as date_parser
#import re
import spacy
import geonamescache


gc= geonamescache.GeonamesCache()


country_set=set(gc.get_countries_by_names().keys())

nlp = spacy.load("en_core_web_sm")


def your_extracting_function(input_file, result_file):
    
    '''
    This function reads the input file (e.g. input.csv)
    and extracts the required information of all given entity mentions.
    The results is saved in the result file (e.g. results.csv)
    '''
    with open(result_file, 'w', encoding='utf8') as fout:
        headers = ['entity','dateOfBirth','nationality','almaMater','awards','workPlaces']
        writer = csv.writer(fout, quoting=csv.QUOTE_MINIMAL)
        writer.writerow(headers)
        
        with open(input_file, 'r', encoding='utf8') as fin:
            reader = csv.reader(fin)

            # skipping header row
            next(reader)
            
            for row in reader:
                entity = row[0]
                abstract = row[1]
                dateOfBirth, nationality, almaMater, awards, workPlace = [], [], [], [], []
                
                '''
                extracting information 
                '''

                dateOfBirth += extract_dob(entity, abstract)
                nationality += extract_nationality(entity, abstract)
                almaMater += extract_almamater(entity, abstract)
                awards += extract_awards(entity, abstract)
                workPlace += extract_workpace(entity, abstract)
                
                writer.writerow([entity, str(dateOfBirth), str(nationality), str(almaMater), str(awards), str(workPlace)])
        
    
'''
date of birth extraction funtion
'''    

def extract_dob(entity, abstract, **kwargs):
    dob = []
    continue_search = True
    birthdate=None
    while (continue_search):
        '''
        ======== to be implemented =========
        birthdayphrase = first text within () 
        birthdate = parse birthdayphrase with date_parser for a relaxed date match (hint: fuzzy match)
        if birthdate found:
            continue_search = False
        else:
            find the next birth date type texts and repeat until end of abstract
        '''
    if birthdate:
        try:
            dob.append(birthdate[0].strftime('%Y-%m-%d'))
        except:
            return []
    return dob

'''
nationality extraction function
'''
def extract_nationality(entity, abstract, **kwargs):
    doc = nlp(abstract)
    '''
    ======== to be implemented =========
    token-level:
        for token in doc:
            token resembles common syntax for nationality (hint: is an adjective, first char is uppercase)
            token resembles frequent nationalities (hint: US, USA, U.S., UK), then return canonical form
            token in pre-defined denonym dictionary (geonamescache), return matching country
    entity level:
        apply same techniques for special labels (hint: GPE)
    '''
    return []
 

'''
alma mater extraction function
'''
def extract_almamater(entity, abstract, **kwargs):
    almaMater = []
    doc = nlp(abstract)
    for ent in doc.ents:
        entity_text= ent.text
        '''
        ======== to be implemented =========
        ## Search for keywords like unversity/college/school/institute in the entity txt 
        ## append to list if keyowrds exist
        almaMater.append(entity_text)
        '''
    return almaMater


'''
awards extracttion function
'''
def extract_awards(entity, abstract, **kwargs):
    awards = []
    doc = nlp(abstract)
    for token in doc:
        '''
        ======== to be implemented =========
        keywords = ['Prize', 'Medal', 'Award', 'Fellow']
        if any(keyword in token for keyword in keywords):
            parse through the token.subtree to get the multi-token award name
                clean unwanted tokens (hint puntuation, numbers,)
            clean the multi-token award name (hint: all capitalized words? keyword still in the string?)
            append the multi-token award name to the list
        '''
    return awards


'''
workplace extraction function
'''
def extract_workpace(entity, abstract, **kwargs):
    workPlace = []
    doc = nlp(abstract)
    for ent in doc.ents:
        entity_text= ent.text
        '''
        ======== to be implemented =========
        keywords = ['University', 'College', 'Laboratory', 'Institute', 'Hospital', 'Medical']
        if any(keyword in entity_text for keyword in keywords):
            clean the entity (hint: remove punctuations, determiners)
            append to the list
        '''
    return workPlace


'''
main function
'''
if __name__ == '__main__':
    if len(sys.argv) != 3:
        raise ValueError('Expected exactly 2 argument: input file and result file')
    your_extracting_function(sys.argv[1], sys.argv[2])
