import sys
import csv
import re
import datefinder
import spacy

##nlp = spacy.load("en_core_web_md")
nlp = spacy.load("en_core_web_sm")

def your_extracting_function(input_file, result_file):
    
    '''
    This function reads the input file (e.g. input.csv)
    and extracts the required information of all given entity mentions.
    The results is saved in the result file (e.g. results.csv)
    '''
    with open(result_file, 'w', encoding='utf8') as fout:
        headers = ['entity','dateOfBirth','nationality','almaMater','awards','workPlaces']
        writer = csv.writer(fout, quoting=csv.QUOTE_MINIMAL)
        writer.writerow(headers)
        
        with open(input_file, 'r', encoding='utf8') as fin:
            reader = csv.reader(fin)

            # skipping header row
            next(reader)
            
            for row in reader:
                entity = row[0]
                abstract = row[1]
                dateOfBirth, nationality, almaMater, awards, workPlace = [], [], [], [], []
                
                '''
                baseline: adding a random value 
                comment this out or remove this baseline 
                '''
                
                #dateOfBirth.append('1961-1-1')
                #nationality.append('United States')
                #almaMater.append('Johns Hopkins University')
                #awards.append('Nobel Prize in Physics')
                #workPlace.append('Johns Hopkins University')
                
                
                '''
                extracting information 
                '''

                dateOfBirth += extract_dob(entity, abstract)
                nationality += extract_nationality(entity, abstract)
                almaMater += extract_almamater(entity, abstract)
                awards += extract_awards(entity, abstract)
                workPlace += extract_workpace(entity, abstract)
                
                writer.writerow([entity, str(dateOfBirth), str(nationality), str(almaMater), str(awards), str(workPlace)])
        
    
'''
date of birth extraction funtion
'''    
def extract_dob(entity, abstract, **kwargs):
    dob = []
    '''
    ======== to be implemented =========
    find all matches of pre-defined date formats using existing libraries (e.g. datefinder) 
    '''
    return dob


'''
nationality extraction function
'''
def extract_nationality(entity, abstract, **kwargs):
    nationality = []
    country_dict={}

    '''
    ======== to be implemented =========
    country_dict = load available denonyms dict
    nation = find first occurence of entity in doc.ents which may equal to nationality (hint: ent label == NORP)
    if nation in country_dict:
        return nationality
    else:
        []
    '''
    return nationality
 
'''
alma mater extraction function
'''
def extract_almamater(entity, abstract, **kwargs):
    almaMater = []
    abstract=clean_sentences(abstract)
    doc = nlp(abstract)
    '''
    ======== to be implemented =========
    for all ent in doc.ents:
        if ent label of a particular type (hint: ORG):
            append ent to almaMater
    '''

    return almaMater


'''
awards extracttion function
'''
def extract_awards(entity, abstract, **kwargs):
    awards = []
    
    abstract=clean_sentences(abstract)
    doc = nlp(abstract)
    '''
    ======== to be implemented =========
    keywords = ['Prize', 'Medal', 'Award', 'Fellow']
    at entity level:
    for all ent indoc.ents:
        if ent label of particular type: (hint: work of art) or any(keyword in ent.text for keyword in keywords):
            append to awards

    at phrase level:
    for chunk in doc.noun_chunks:
        if any(keyword in chunk.text for keyword in keywords):
            append to awards
    '''
    return awards


'''
workplace extraction function
'''
def extract_workpace(entity, abstract, **kwargs):
    workPlace = []
    abstract=clean_sentences(abstract)
    doc = nlp(abstract)
    '''
    ======== to be implemented =========
    keywords = ['University', 'College', 'Laboratory', 'Institute', 'Hospital', 'Medical']
    at entity level:
    for all ent indoc.ents:
        if ent label of particular type: (hint: ORG) or any(keyword in ent.text for keyword in keywords):
            append to workPlace
    '''
    return workPlace

'''
abstract cleaning function
'''
def clean_sentences(sentence):   
    sentence=re.sub('\((?:[^)(]|\([^)(]*\))*\)', '',sentence) #Removing nested ()
    sentence=re.sub('"', '',sentence) #Removing " "
    return sentence 

'''
main function
'''
if __name__ == '__main__':
    if len(sys.argv) != 3:
        raise ValueError('Expected exactly 2 argument: input file and result file')
    your_extracting_function(sys.argv[1], sys.argv[2])
